/* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
/* Balsa E-Mail Client
 *
 * Copyright (C) 1997-2000 Stuart Parmenter and others,
 *                         See the file AUTHORS for a list.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option) 
 * any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 */
/*
 * filter-private.h
 *
 * private filter defninitions
 */

#ifndef __FILTER_PRIVATE_H__
#define __FILTER_PRIVATE_H__

#ifdef HAVE_PCRE
#  include <pcreposix.h>
#else
#  include <sys/types.h>
#  include <regex.h>
#endif

#include "libmutt/mutt.h"
#include "libmutt/mutt_regex.h"

/* regex options */
#define FILTER_REGCOMP       (REG_NEWLINE | REG_NOSUB | REG_EXTENDED)
#define FILTER_REGEXEC       0

/* Conditions definitions */

/*  match flags */
#define CONDITION_EMPTY         0       /* for initialization */
#define CONDITION_MATCH_TO      1<<0	/* match in the To: field */
#define CONDITION_MATCH_FROM    1<<1	/* match in the From: field */
#define CONDITION_MATCH_SUBJECT 1<<2	/* match in the Subject field */
#define CONDITION_MATCH_CC      1<<3	/* match in the cc: field */
#define CONDITION_MATCH_BODY    1<<7	/* match in the body */

/* match_fields macros */
#define CONDITION_SETMATCH(x, y) \
          ((((LibBalsaCondition*)(x))->match_fields) |= (y))
#define CONDITION_CLRMATCH(x, y) \
          ((((LibBalsaCondition*)(x))->match_fields) &= ~(y))
#define CONDITION_CHKMATCH(x, y) \
          ((((LibBalsaCondition*)(x))->match_fields) & (y))

/* regex struct */
typedef struct _LibBalsaConditionRegex {
    gchar *string;
    regex_t *compiled;
} LibBalsaConditionRegex;

/* Filter defintions */

/* filter flags */
#define FILTER_EMPTY         0	/* for clearing bitfields */

#define FILTER_VALID         1<<1	/* ready to filter (eg regex strings 
					   have been compiled with regcomp(), with no errors...) */					
#define FILTER_COMPILED      1<<2	/* the filter needs to be compiled (ie there are uncompiled regex) */
#define FILTER_SOUND         1<<4	/* play a sound when matches */
#define FILTER_POPUP         1<<5	/* popup text when matches */

/* flag operation macros */
#define FILTER_SETFLAG(x, y) ((((LibBalsaFilter*)(x))->flags) |= (y))
#define FILTER_CLRFLAG(x, y) ((((LibBalsaFilter*)(x))->flags) &= ~(y))
#define FILTER_CHKFLAG(x, y) ((((LibBalsaFilter*)(x))->flags) & (y))

#endif				/* __FILTER_PRIVATE_H__ */
